/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "feigenbaum_fileaccess.h"

#include <qdatastream.h>
#include <qfile.h>

Feigenbaum::FileAccess::FileAccess()
	: QObject()
{
}

Feigenbaum::FileAccess::~FileAccess()
{
}

Handle<Feigenbaum::CalcInputData> Feigenbaum::FileAccess::open( QString fileName )
{
	DoubleRect coords;
	IntSize picSize;
	unsigned int maxIterations;
	unsigned int paintIterations;
	double startValue;

	QFile file( fileName );
	if( ! file.open( IO_ReadOnly ) ) return 0;

	QDataStream ds( &file );

	// read signature to be sure we have a ChaosExplorer file
	char c_str[14];
	ds.readRawBytes( c_str, 14 );
	QString str( c_str );
	if( str != "CHAOSEXPLORER" ) return 0;

	// read file format version number (not used yet)
	int ver_maj, ver_min, ver_sub;
	ds >> ver_maj >> ver_min >> ver_sub;

	// read doc type
	int doc_type;
	ds >> doc_type;
	if( doc_type != 1 ) return 0; // only 1 corresponds to feigenbaum

	// read "the rest"
	ds >> coords.tl.x >> coords.tl.y;
	ds >> coords.br.x >> coords.br.y;
	ds >> picSize.width >> picSize.height;
	ds >> maxIterations;
	ds >> paintIterations;
	ds >> startValue;

	file.close();

	m_fileName = fileName;

	CalcInputData* cid = new CalcInputData();
	cid->setCoords( coords );
	cid->setPicSize( picSize );
	cid->setMaxIterations( maxIterations );
	cid->setPaintIterations( paintIterations );
	cid->setStartValue( startValue );

	return cid;
}

bool Feigenbaum::FileAccess::save( QString fileName, Handle<CalcInputData> input )
{
	if( fileName.isEmpty() )
	{
		if( m_fileName.isEmpty() )
			return false;
		else
			fileName = m_fileName;
	}

	QFile file( fileName );
	if( ! file.open( IO_WriteOnly ) ) return false;

	QDataStream ds( &file );
	ds.writeRawBytes( "CHAOSEXPLORER\0", 14 ); // file type signature
	ds << 1 << 0 << 0; // file format version
	ds << 1; // type feigenbaum
	ds << input->coords().tl.x << input->coords().tl.y;
	ds << input->coords().br.x << input->coords().br.y;
	ds << input->picSize().width << input->picSize().height;
	ds << input->maxIterations();
	ds << input->paintIterations();
	ds << input->startValue();

	file.close();

	m_fileName = fileName;

	return true;
}

const QString& Feigenbaum::FileAccess::fileName() const
{
	return m_fileName;
}

bool Feigenbaum::FileAccess::unsaved() const
{
	return m_unsaved;
}

