/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "feigenbaum_propdialog.h"

Feigenbaum::PropDialog::PropDialog( QWidget* parent, const char* name )
	: QDialog( parent, name, true )
{

	setCaption( "Feigenbaum-Einstellungen" );

	QVBoxLayout* vbox = new QVBoxLayout( this, 10 );

	QTabWidget* tabwidget = new QTabWidget( this );
	vbox->addWidget( tabwidget );

	// create tab 1

	QWidget* calc = new QWidget( this );

	QVBoxLayout* calc_vbox = new QVBoxLayout( calc, 15 );

	QGridLayout* calc_grid = new QGridLayout( calc_vbox, 3, 2, 15 );
	m_maxIterations = new QSpinBox( 1, 4000, 1, calc );
	m_paintIterations = new QSpinBox( 1, 4000, 1, calc );
	m_startValue = new QLineEdit( calc );
	calc_grid->addWidget( new QLabel( "Iterationen:", calc ), 0, 0 );
	calc_grid->addWidget( m_maxIterations, 0, 1 );
	calc_grid->addWidget( new QLabel( "Dargestellte Iterationen:", calc ), 1, 0 );
	calc_grid->addWidget( m_paintIterations, 1, 1 );
	calc_grid->addWidget( new QLabel( "Startwert fr Iteration:", calc ), 2, 0 );
	calc_grid->addWidget( m_startValue, 2, 1 );

	calc_vbox->addStretch();

	connect( m_maxIterations, SIGNAL( valueChanged( int ) ), SLOT( maxIterChanged( int ) ) );

	// create tab 2

	QWidget* coord = new QWidget( this );

	QVBoxLayout* coord_vbox = new QVBoxLayout( coord, 15 );

	QGroupBox* top_left = new QGroupBox( "Links oben", coord );
	coord_vbox->addWidget( top_left );
	QGridLayout* top_left_grid = new QGridLayout( top_left, 2, 2, 15 );
	m_x1 = new QLineEdit( top_left );
	m_x1->setValidator( new QDoubleValidator( m_x1 ) );
	m_y1 = new QLineEdit( top_left );
	m_y1->setValidator( new QDoubleValidator( m_y1 ) );
	top_left_grid->addWidget( new QLabel( "X:", top_left ), 0, 0 );
	top_left_grid->addWidget( m_x1, 0, 1 );
	top_left_grid->addWidget( new QLabel( "Y:", top_left ), 1, 0 );
	top_left_grid->addWidget( m_y1, 1, 1 );

	QGroupBox* bottom_right = new QGroupBox( "Rechts unten", coord );
	coord_vbox->addWidget( bottom_right );
	QGridLayout* bottom_right_grid = new QGridLayout( bottom_right, 2, 2, 15 );
	m_x2 = new QLineEdit( bottom_right );
	m_x2->setValidator( new QDoubleValidator( m_x2 ) );
	m_y2 = new QLineEdit( bottom_right );
	m_y2->setValidator( new QDoubleValidator( m_y2 ) );
	bottom_right_grid->addWidget( new QLabel( "X:", bottom_right ), 0, 0 );
	bottom_right_grid->addWidget( m_x2, 0, 1 );
	bottom_right_grid->addWidget( new QLabel( "Y:", bottom_right ), 1, 0 );
	bottom_right_grid->addWidget( m_y2, 1, 1 );

	coord_vbox->addStretch();

	// add tabs

	tabwidget->addTab( calc, "Berechnung" );
	tabwidget->addTab( coord, "Koordinaten" );

	// add "ok"/"cancel" buttons

	QHBoxLayout* hbox = new QHBoxLayout( vbox, 10 );

	QPushButton* ok = new QPushButton( "&OK", this );
	QPushButton* cancel = new QPushButton( "Ab&brechen", this );
	//QPushButton* apply = new QPushButton( "&bernehmen", this );
	connect( ok, SIGNAL( clicked() ), SLOT( accept() ) );
	connect( cancel, SIGNAL( clicked() ), SLOT( reject() ) );
	hbox->addStretch();
	hbox->addWidget( ok );
	hbox->addWidget( cancel );
	//hbox->addWidget( apply );

}

Feigenbaum::PropDialog::~PropDialog()
{
}

void Feigenbaum::PropDialog::setMaxIterations( int new_maxiterations )
{
	m_maxIterations->setValue( new_maxiterations );
}

int Feigenbaum::PropDialog::maxIterations() const
{
	return m_maxIterations->value();
}

void Feigenbaum::PropDialog::setPaintIterations( int new_paintiterations )
{
	m_paintIterations->setValue( new_paintiterations );
}

int Feigenbaum::PropDialog::paintIterations() const
{
	return m_paintIterations->value();
}

void Feigenbaum::PropDialog::setStartValue( double startvalue )
{
	m_startValue->setText( QString( "%1" ).arg( startvalue ) );
}

double Feigenbaum::PropDialog::startValue() const
{
	double new_startValue;
	QTextStream stream( m_startValue->text(), IO_ReadOnly );
	stream >> new_startValue;
	if( new_startValue > 1.0 ) new_startValue = 1.0;
	if( new_startValue < 0.0 ) new_startValue = 0.0;
	return new_startValue;
}

void Feigenbaum::PropDialog::setCoords( const DoubleRect& coords )
{
	m_x1->setText( QString( "%1" ).arg( coords.tl.x, 0, 'f', 15 ) );
	m_y1->setText( QString( "%1" ).arg( coords.tl.y, 0, 'f', 15 ) );
	m_x2->setText( QString( "%1" ).arg( coords.br.x, 0, 'f', 15 ) );
	m_y2->setText( QString( "%1" ).arg( coords.br.y, 0, 'f', 15 ) );
}

DoubleRect Feigenbaum::PropDialog::coords() const
{
	double x1, y1, x2, y2;
	QTextStream* stream;
	stream = new QTextStream( m_x1->text(), IO_ReadOnly );
	*stream >> x1;
	delete stream;
	stream = new QTextStream( m_y1->text(), IO_ReadOnly );
	*stream >> y1;
	delete stream;
	stream = new QTextStream( m_x2->text(), IO_ReadOnly );
	*stream >> x2;
	delete stream;
	stream = new QTextStream( m_y2->text(), IO_ReadOnly );
	*stream >> y2;
	delete stream;
	return DoubleRect( x1, y1, x2, y2 );
}

void Feigenbaum::PropDialog::maxIterChanged( int value )
{
	m_paintIterations->setMaxValue( value );
}
