/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#if !defined(MANDELBROT_H)
#define MANDELBROT_H

#include "docinterface.h"
#include "mandelbrot_propdialog.h"
#include "mandelbrot_calcinputdata.h"
#include "mandelbrot_calc.h"
#include "mandelbrot_calcresult.h"
#include "mandelbrot_fileaccess.h"
#include "mandelbrot_actioncollection.h"

namespace Mandelbrot
{

	class Document : public DocInterface
	{
		Q_OBJECT
	public:
		Document();
		~Document();

		bool openFile( QString fileName );
		bool saveFile( QString fileName = "" );
		bool exportBitmap( QString fileName );

		QString fileName() const;

		void printBitmap( const QPrinter* printer );

		void setPicSize( const IntSize& size );
		const IntSize& picSize() const;

		void setCoords( const DoubleRect& coords );
		const DoubleRect& coords() const;

		void startCalc();
		void stopCalc();
		bool calcing() const;

		void undo();
		void redo();
		bool can_undo() const;
		bool can_redo() const;
		QString undo_descr() const;
		QString redo_descr() const;

		QPixmap& pixmap();

		void showProperties( QWidget* parent = 0 );

	private:
		Handle<CalcInputData> m_input;
		Handle<Calc> m_calc;
		Handle<CalcResult> m_result;
		Handle<FileAccess> m_fileaccess;
		Handle<ActionCollection> m_actioncollection;
	};

}

#endif // !defined(MANDELBROT_H)

