/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#ifndef MANDELBROT_CALC_H
#define MANDELBROT_CALC_H

#include <qthread.h>
#include <qobject.h>
#include <complex>
using std::complex;
using std::abs;
using std::pow;
typedef complex<double> compl;

#include "vartypes.h"
#include "mandelbrot_calcinputdata.h"
#include "mandelbrot_calcresult.h"

namespace Mandelbrot
{
	class Calc : public QObject, public QThread
	{
		Q_OBJECT
	public:
		Calc( Handle<CalcInputData> input, Handle<CalcResult> output );
		~Calc();

		void stop();

		bool calcing();

	signals:
		void started();
		void stopped();
		void finished();
		void rectPainted( const IntRect& );

	private slots:
		void colorsChanged();

	private:
		void run();

		void calcUnOptimized();
		void calcInterleaved();
		void calcSquares( const IntRect& rect );
		void calcBoundaryTraced();

		int iterateComplex( const compl& pos );
		void calcPalette();

		bool m_calcing;
		Handle<CalcInputData> m_input;
		Handle<CalcResult> m_output;

	};
}

#endif
